@@ -360,23 +360,17 @@ build_fault_response(Code, ParseString, ParseArgs) ->
     FaultString = "Error " ++ integer_to_list(Code) ++ "\n"
         ++ lists:flatten(io_lib:format(ParseString, ParseArgs)),
     ?WARNING_MSG(FaultString, []),
-    {false, {response, {fault, Code, FaultString}}}.
+    {false, {response, {fault, Code, list_to_binary(FaultString)}}}.
 
 do_command(AccessCommands, Auth, Command, AttrL, ArgsF,
 	   ResultF) ->
     ArgsFormatted = format_args(AttrL, ArgsF),
-    AuthBin = convert_auth(Auth),
     Result =
-	ejabberd_commands:execute_command(AccessCommands, AuthBin,
+	ejabberd_commands:execute_command(AccessCommands, Auth,
 					  Command, ArgsFormatted),
     ResultFormatted = format_result(Result, ResultF),
     {command_result, ResultFormatted}.
 
-convert_auth(noauth) ->
-    noauth;
-convert_auth({UserT, ServerT, PasswordT}) ->
-    {list_to_binary(UserT), list_to_binary(ServerT), list_to_binary(PasswordT)}.
-
 %%-----------------------------
 %% Format arguments
 %%-----------------------------
